--[[
Version History:
v1.5a
* Updated the TOC file for 4.3
* Fixed a bug related to the total avoidance going negative
* Fixed a bug showing a little green bar when the actual total avoidance is negative
* No longer shows the "Avoidance + Block" line in the tooltip when the block chance is 0%

v1.5
* Some tooltip and options tweaks
* The tooltip now updates continuosly as the combat table bar updates
* The target level input box does not require you to press Enter anymore
* Added an option to always show the tooltip (tooltip anchoring is not always the best though, use with caution)
* Added a "Cancel" button to the options frame, which will roll-back all changes
* Added the version number to the options menu

v1.4
* Some minor fixes here and there (as usual)
* The Hold The Line talent is now taken into account for the warrior's critical block chance
* Slightly reworked options menu

v1.3a
* Fixed a minor error in incoming damage calculation, it counted critical hits twice instead of just once

v1.3
* More minor code optimizations and simplifications
* Some small color changes
* The zero-value combat table entries are now not shown in the tooltip
* Fixed the localizing issue with the "Shields" subtype that prevented the block chance of calculating on non-EN clients
* Added blocked amount and Warrior's Critical Block calculations
* Added damage mitigation calculations, including the blocked amount and armor damage reduction, as well as total incoming damage both before and after armor is calculated

v1.2
* Minor fixes
* Adds the 2% additional miss racial for the Night Elves
* Checks for the correct stance/presence for Warriors/Death Knights when calculating the crit chance

v1.1
* Added the option to change the default target level, 0 for automatic (uses your target or your own level if you have no target), or it can be set from 1 to 100
* Updated the interface version in the TOC file to match WoW version 4.2.2

v1.0
* Initial Release
]]

local VCT = {}
VCT.TimeSinceLastUpdate = 0
VCT.UpdateInterval = 0.3
VCT.Textures = {}
VCT_Var = {}

local VCT_DefaultVars = {
	RelPoint = "CENTER",
	X = 0,
	Y = 0,	
	Height = 20,
	Width = 208,
	Hidden = false,
	Locked = false,
	AlwaysShowTooltip = false,
	TargetLevel = 0
}


SLASH_VCT1 = "/vct"
local CTBlockValue, CTCBlockChance, CTMiss, CTDodge, CTParry, CTBlock, CTHit, CTCrit, CTCrush, VCTFill, msg1, msg2, LevDiff=0, CTMitigation, CTArmorDR, CTFinalDamage, locshields

function VCT:LoadDefaults(saved, defaults)
	setmetatable(saved, {__index = defaults})
	for i, v in pairs(saved) do
		if type(v) == "table" and not getmetatable(v) then
			VCT:LoadDefaults(v, defaults[i])
		end
	end
end

function VCT:Loadup()
	local VCTDDownItems = {{name="Lock/Unlock", func=function () if VCT_Var.Locked == true then VCT_Var.Locked = false else VCT_Var.Locked = true end end}, {name="Hide", func= function() VCT.MainFrame:Hide() VCT_Var.Hidden = true print("Visual Combat Table is hidden. Use the command \"/vct toggle\" to show it again") end}, {name="Options", func=function() VCT.OptionsFrame:Show() end}, {name="Cancel", func=nil}}
	SlashCmdList["VCT"] = function(msg) VCT:SlashHandler(msg) end
	_, VCT.PlayerClass = UnitClass("player")	
	_, VCT.PlayerRace = UnitRace("player")	
	_,_,_,_,_,locShields = GetAuctionItemSubClasses(2)
	CreateFrame("GameTooltip", "VCTTooltip", UIParent, "GameTooltipTemplate")
	VCT.MainFrame:SetHeight(VCT_Var.Height)
	VCT.MainFrame:SetWidth(VCT_Var.Width)
	VCT.MainFrame:SetMovable(true)
	VCT.MainFrame:EnableMouse(true)	
	VCT.MainFrame:SetPoint(VCT_Var.RelPoint, UIParent, VCT_Var.X, VCT_Var.Y)
	VCT.MainFrame:SetScript("OnMouseDown", function(self, button)
		if button == "LeftButton" then
			if VCT_Var.Locked == false then
				VCT.MainFrame:StartMoving()
			end
		elseif button == "RightButton" then
			ToggleDropDownMenu(1, nil, VCT.DropDownMenu, VCT.MainFrame, 0, 0);
		end
	end)
	VCT.MainFrame:SetScript("OnMouseUp", function(self, button)
		if button == "LeftButton" then
			VCT.MainFrame:StopMovingOrSizing()
			_,_, VCT_Var.RelPoint, VCT_Var.X, VCT_Var.Y = VCT.MainFrame:GetPoint()
		end	
	end)
	
	VCT.DropDownMenu = CreateFrame("FRAME", "OptionsDropDown", VCT.MainFrame, "UIDropDownMenuTemplate")
	VCT.DropDownMenu:SetID(1)
	VCT.DropDownMenu:ClearAllPoints()
	VCT.DropDownMenu:Show()

	local function VCTDropdownInit(self, level)
		local info = UIDropDownMenu_CreateInfo()
		for i=1, #VCTDDownItems do
			info = UIDropDownMenu_CreateInfo()
			info.text = VCTDDownItems[i].name
			info.value = i
			info.notCheckable = 1
			info.func = VCTDDownItems[i].func
			UIDropDownMenu_AddButton(info, level)
		end
	end
	UIDropDownMenu_Initialize(VCT.DropDownMenu, VCTDropdownInit, "MENU")

	
	VCT.Textures.GTexture = VCT.MainFrame:CreateTexture("", "BACKGROUND")
	VCT.Textures.GTexture:SetTexture(0,1,0,1)
	VCT.Textures.YTexture = VCT.MainFrame:CreateTexture("", "BACKGROUND")
	VCT.Textures.YTexture:SetTexture(1,1,0,1)
	VCT.Textures.OTexture = VCT.MainFrame:CreateTexture("", "BACKGROUND")
	VCT.Textures.OTexture:SetTexture(1,0.5,0,1)
	VCT.Textures.RTexture = VCT.MainFrame:CreateTexture("", "BACKGROUND")
	VCT.Textures.RTexture:SetTexture(1,0,0,1)
	VCT.Textures.PTexture = VCT.MainFrame:CreateTexture("", "BACKGROUND")
	VCT.Textures.PTexture:SetTexture(1,0,1,1)
	VCT.MainFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border.blp", tile = true, tileSize = 16, edgeSize = 16, insets = { left = 3, right = 3, top = 3, bottom = 3}})
	VCT.MainFrame:SetBackdropColor(0,0,0,0)
	if VCT_Var.Hidden then VCT.MainFrame:Hide() else VCT.MainFrame:Show() end

	VCT.MainFrame:SetScript("OnEnter", VCTOnEnter)
	VCT.MainFrame:SetScript("OnLeave", VCTOnLeave)
		
	VCT.MainFrame:SetScript("OnUpdate", function (self, elapsed)
		VCT.TimeSinceLastUpdate = VCT.TimeSinceLastUpdate + elapsed
		while (VCT.TimeSinceLastUpdate > VCT.UpdateInterval) do
			VCT:UpdateFrame()
			VCT.TimeSinceLastUpdate = VCT.TimeSinceLastUpdate - VCT.UpdateInterval
		end
	end)

	local VCTHeightBk, VCTWidthBk, VCTLevelBk, VCTASTT
	
	VCT.OptionsFrame = CreateFrame("FRAME", nil, VCT.MainFrame)
	VCT.OptionsFrame:Hide()
	VCT.OptionsFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border.blp", tile = true, tileSize = 16, edgeSize = 16, insets = { left = 3, right = 3, top = 3, bottom = 3}})
	VCT.OptionsFrame:SetBackdropColor(0,0,0,1)
	VCT.OptionsFrame:SetHeight(170)
	VCT.OptionsFrame:SetWidth(380)
	VCT.OptionsFrame:SetMovable(true)
	VCT.OptionsFrame:SetScript("OnMouseDown", function()	VCT.OptionsFrame:StartMoving() end)
	VCT.OptionsFrame:SetScript("OnMouseUp", function() VCT.OptionsFrame:StopMovingOrSizing() end)
	VCT.OptionsFrame:SetScript("OnShow", function()
											VCTHeightBk = VCT_Var.Height
											VCTWidthBk = VCT_Var.Width
											VCTLevelBk = VCT_Var.TargetLevel
											VCTASTT = VCT_Var.AlwaysShowTooltip
										end)
		
	VCT.lbHeightSlider = VCT.OptionsFrame:CreateFontString(nil, "OVERLAY")
	VCT.lbHeightSlider:SetFont("Fonts\\FRIZQT__.TTF", 12, "THICKOUTLINE")
	VCT.lbHeightSlider:SetText("Bar Height:")
	VCT.lbHeightSlider:SetPoint("TOPLEFT", VCT.OptionsFrame, 3, -15)
		
	VCT.HeightSlider = CreateFrame("Slider", "VCTHeightSlider", VCT.OptionsFrame, "OptionsSliderTemplate")
	VCT.HeightSlider:SetPoint("TOPLEFT", VCT.OptionsFrame, VCT.lbHeightSlider:GetWidth()+20, -10)
	VCT.HeightSlider:SetWidth(270)
	VCT.HeightSlider:SetMinMaxValues(20, 300)
	VCT.HeightSlider:SetValue(VCT_Var.Height)
	VCT.HeightSlider:SetValueStep(5)
	VCTHeightSliderLow:SetText("20")
	VCTHeightSliderHigh:SetText("300")
	VCT.HeightSlider:SetScript("OnValueChanged", function()
													VCT_Var.Height=VCT.HeightSlider:GetValue()
													VCT.MainFrame:SetHeight(VCT.HeightSlider:GetValue())
													VCT:UpdateFrame()
												end)
	VCT.HeightSlider:SetScript("OnShow", function() VCT.HeightSlider:SetValue(VCT_Var.Height) end)
	
	VCT.lbWidthSlider = VCT.OptionsFrame:CreateFontString(nil, "OVERLAY")
	VCT.lbWidthSlider:SetFont("Fonts\\FRIZQT__.TTF", 12, "THICKOUTLINE")
	VCT.lbWidthSlider:SetText("Bar Width:")
	VCT.lbWidthSlider:SetPoint("TOPLEFT", VCT.OptionsFrame, 3, -45)
		
	VCT.WidthSlider = CreateFrame("Slider", "VCTWidthSlider", VCT.OptionsFrame, "OptionsSliderTemplate")
	VCT.WidthSlider:SetPoint("TOPLEFT", VCT.OptionsFrame, VCT.lbHeightSlider:GetWidth()+20, -40)
	VCT.WidthSlider:SetWidth(270)
	VCT.WidthSlider:SetMinMaxValues(20, 800)
	VCT.WidthSlider:SetValue(VCT_Var.Width)
	VCT.WidthSlider:SetValueStep(5)
	VCTWidthSliderLow:SetText("20")
	VCTWidthSliderHigh:SetText("800")
	VCT.WidthSlider:SetScript("OnValueChanged", function()
													VCT_Var.Width=VCT.WidthSlider:GetValue()
													VCT.MainFrame:SetWidth(VCT.WidthSlider:GetValue())
													VCT:UpdateFrame()
												end)
	VCT.WidthSlider:SetScript("OnShow", function() VCT.WidthSlider:SetValue(VCT_Var.Width) end)
	
	VCT.TargetLevelBox = CreateFrame("EditBox", "TargetLevel", VCT.OptionsFrame, "InputBoxTemplate")
	VCT.TargetLevelBox:SetPoint("TOPLEFT", VCT.OptionsFrame, 340, -70)
	VCT.TargetLevelBox:SetHeight(20)
	VCT.TargetLevelBox:SetWidth(30)
	VCT.TargetLevelBox:SetAutoFocus(false)
	VCT.TargetLevelBox:SetNumeric(true)
	VCT.TargetLevelBox:SetScript("OnChar", function() if (VCT.TargetLevelBox:GetNumber()>=0) and (VCT.TargetLevelBox:GetNumber()<=100) then VCT_Var.TargetLevel=VCT.TargetLevelBox:GetNumber() end --[[VCT.TargetLevelBox:HighlightText()]] end)
	VCT.TargetLevelBox:SetScript("OnEscapePressed", function() VCT.TargetLevelBox:ClearFocus() end)
	VCT.TargetLevelBox:SetScript("OnShow", function() VCT.TargetLevelBox:SetNumber(VCT_Var.TargetLevel) end)
		
	VCT.lbTargetLevelBox = VCT.OptionsFrame:CreateFontString(nil, "OVERLAY")
	VCT.lbTargetLevelBox:SetFont("Fonts\\FRIZQT__.TTF", 12, "THICKOUTLINE")
	VCT.lbTargetLevelBox:SetText("Target Level (1-100, 0 for automatic):")
	VCT.lbTargetLevelBox:SetPoint("TOPLEFT", VCT.OptionsFrame, 3, -75)
		
	VCT.ASTTCheckBox = CreateFrame("CheckButton", nil, VCT.OptionsFrame, "ChatConfigCheckButtonTemplate")
	VCT.ASTTCheckBox:SetPoint("TOPLEFT", 5, -93)
	VCT.ASTTCheckBox:SetScript("OnShow", function() if VCT_Var.AlwaysShowTooltip then VCT.ASTTCheckBox:SetChecked() else VCT.ASTTCheckBox:SetChecked(false) end end)
	VCT.ASTTCheckBox:SetScript("OnClick", function()
											if VCT.ASTTCheckBox:GetChecked() then
												VCT_Var.AlwaysShowTooltip = true
												VCTOnEnter()
											else
												VCT_Var.AlwaysShowTooltip = false
												VCTTooltip:Hide()
											end
										end)
	
	VCT.lbASTTCheckBox = VCT.OptionsFrame:CreateFontString(nil, "OVERLAY")
	VCT.lbASTTCheckBox:SetFont("Fonts\\FRIZQT__.TTF", 12, "THICKOUTLINE")
	VCT.lbASTTCheckBox:SetText("Always show the tooltip")
	VCT.lbASTTCheckBox:SetPoint("TOPLEFT", VCT.OptionsFrame, 30, -100)
		
	VCT.OptionsLabel = 	VCT.OptionsFrame:CreateFontString(nil, "OVERLAY")
	VCT.OptionsLabel:SetFont("Fonts\\FRIZQT__.TTF", 9, "THICKOUTLINE")
	VCT.OptionsLabel:SetText("Visual Combat Table v1.5a")
	VCT.OptionsLabel:SetPoint("BOTTOMRIGHT", VCT.OptionsFrame, -220, 10)
	VCT.OptionsLabel:SetTextColor(0, 1, 0, 1)
		
	VCT.DoneButton = CreateFrame("Button", nil, VCT.OptionsFrame, "UIPanelButtonTemplate")
	VCT.DoneButton:SetText("Done")
	VCT.DoneButton:SetPoint("BOTTOMRIGHT", -75, 10)
	VCT.DoneButton:SetHeight(35)
	VCT.DoneButton:SetWidth(55)
	VCT.DoneButton:SetScript("OnClick", function() VCT.OptionsFrame:Hide() end)

	VCT.CancelButton = CreateFrame("Button", nil, VCT.OptionsFrame, "UIPanelButtonTemplate")
	VCT.CancelButton:SetText("Cancel")
	VCT.CancelButton:SetPoint("BOTTOMRIGHT", -10, 10)
	VCT.CancelButton:SetHeight(35)
	VCT.CancelButton:SetWidth(55)
	VCT.CancelButton:SetScript("OnClick", function()
											VCT_Var.Height = VCTHeightBk
											VCT.MainFrame:SetHeight(VCT_Var.Height)
											VCT_Var.Width = VCTWidthBk
											VCT.MainFrame:SetWidth(VCT_Var.Width)
											VCT_Var.TargetLevel = VCTLevelBk
											VCT_Var.AlwaysShowTooltip = VCTASTT
											if VCTTooltip:IsVisible() and not VCTASTT then VCTTooltip:Hide() end
											VCT:UpdateFrame()
											VCT.OptionsFrame:Hide()
											end)
	
	VCT.OptionsFrame:SetPoint("CENTER", UIParent, 0, 0)
end

function VCT:SlashHandler(msg)
	msg1, msg2 = msg:match("^(%S*)%s*(.-)$")
	if msg1=="toggle" then
		if VCT.MainFrame:IsVisible() then
				VCT.MainFrame:Hide()
				print("Visual Combat Table is hidden. Use the command \"/vct toggle\" to show it again")
				VCT_Var.Hidden = true
			else
				VCT.MainFrame:Show()
				VCT_Var.Hidden = false
		end
	elseif msg1=="height" then
		if tonumber(msg2)>=20 and tonumber(msg2)<=300 then
			VCT.MainFrame:SetHeight(tonumber(msg2))
			VCT_Var.Height=tonumber(msg2)
			VCT:UpdateFrame()
		end
	elseif msg1=="width" then
		if tonumber(msg2)>=30 then
			VCT.MainFrame:SetWidth(tonumber(msg2))
			VCT_Var.Width=tonumber(msg2)
			VCT:UpdateFrame()
		end
	elseif msg1=="lock" then
		VCT_Var.Locked = true
	elseif msg1=="ast" then
		if msg2=="on" then
			VCT_Var.AlwaysShowTooltip = true
			VCTOnEnter()
		elseif msg2=="off" then
			VCT_Var.AlwaysShowTooltip = false
			VCTTooltip:Hide()
	end
	elseif msg1=="unlock" then
		VCT_Var.Locked = false
	elseif msg1=="setlevel" then
		if tonumber(msg2)>=0 and tonumber(msg2)<=100 then
			VCT_Var.TargetLevel=tonumber(msg2)
			VCT:UpdateFrame()
		end
	else
		print("Visual Combat Table v1.5a")
		print("Created by BlaDeR (aka Fangion of EU-Stormrage)")
		print("Use /vct toggle to show/hide the bar")
		print("Use /vct height xx to change the bar height (20-300)")
		print("Use /vct width xx to change the bar width (minimum of 30)")
		print("Use /vct setlevel xx to change the lock the default target level, use 0 if you want it to auto-adjust for your target or yourself")
		print("Use /vct ast on|off to choose whether to always show the tooltip or not")
		print("Use /vct lock or /vct unlock to lock/unlock the frame")
	end
end

function VCTOnEnter()
	VCTTooltip:ClearLines()
	VCTTooltip:SetOwner(VCT.MainFrame, "ANCHOR_TOPLEFT")
	VCTTooltip:SetText(format("Defensive Combat Table against a level %u attacker:", UnitLevel("player")+LevDiff))
	VCTTooltip:AddLine(format("Miss: %.2f%%\nDodge: %.2f%% \nParry: %.2f%%", CTMiss, CTDodge, CTParry), 0.85, 0.85, 0.7)
	VCTTooltip:AddLine(format("Total Avoidance: %.2f%%", CTMiss+CTDodge+CTParry), 0, 1, 0)
	if CTBlock>0 then VCTTooltip:AddLine(format("Blocked Hit: %.2f%%", CTBlock), 1, 1, 0) end
	if (CTBlock>0 and VCT.PlayerClass == "WARRIOR" and GetPrimaryTalentTree()==3) then
		VCTTooltip:AddLine(format("Of it:"), 1, 1, 0)
		VCTTooltip:AddLine(format("- Normally Blocked Hit: %.2f%%", CTBlock/100*(100-CTCBlockChance)), 1, 1, 0)
		VCTTooltip:AddLine(format("- Critically Blocked Hit: %.2f%%", CTBlock*CTCBlockChance/100), 1, 1, 0)
	end
	if CTBlock>0 and (CTMiss+CTDodge+CTParry)>0 then VCTTooltip:AddLine(format("Avoidance + Block: %.2f%%", CTBlock+CTMiss+CTDodge+CTParry), 0.85, 0.85, 0.7) end
	if CTCrit>0 then VCTTooltip:AddLine(format("Critical Hit: %.2f%%", CTCrit), 1, 0, 0) end
	if CTCrush>0 then VCTTooltip:AddLine(format("Crushing Blow: %.2f%%", CTCrush), 1, 0, 1) end
	if CTHit>0 then VCTTooltip:AddLine(format("Normal Hit: %.2f%%", CTHit), 1, 0.5, 0) end
	VCTTooltip:AddLine("Damage Reduction (Mitigation):")
	if (VCT.PlayerClass == "WARRIOR" or VCT.PlayerClass == "PALADIN" or VCT.PlayerClass == "SHAMAN") and CTBlock>0 then
		if (VCT.PlayerClass == "WARRIOR" and GetPrimaryTalentTree()==3) then
			VCTTooltip:AddLine(format("Average DR on Blocked Hit incl. Critical Block: %.2f%%", (100-CTCBlockChance)/100*30+(CTCBlockChance)*60/100), 0.85, 0.85, 0.7)
		else
			VCTTooltip:AddLine(format("Damage Reduction on Blocked Hit: %.2f%%", CTBlockValue*100), 0.85, 0.85, 0.7)
		end
	end
	VCTTooltip:AddLine(format("Incoming Damage Before Armor: %.2f%%", CTMitigation), 0.85, 0.85, 0.7)
	VCTTooltip:AddLine(format("Damage Reduced By Armor: %.2f%%", CTArmorDR*100), 0.85, 0.85, 0.7)
	VCTTooltip:AddLine(format("Total Incoming Damage: %.2f%%", (1-CTArmorDR)*CTMitigation), 0.68, 0.69, 0.1)
	VCTTooltip:Show()
end

function VCTOnLeave()
	if not VCT_Var.AlwaysShowTooltip then VCTTooltip:Hide() end
end

function VCT:LowerCrit()
	local rank, active
	if VCT.PlayerClass=="PALADIN" then
		_,_,_,_,rank = GetTalentInfo(2, 8)
		return rank*2
	elseif VCT.PlayerClass=="WARRIOR" then
		_, _, active = GetShapeshiftFormInfo(2);
		if active then
			_,_,_,_,rank = GetTalentInfo(3, 10)
			return rank*3
		end
		return 0
	elseif VCT.PlayerClass=="DRUID" then
		_,_,_,_,rank = GetTalentInfo(2, 11)
		return rank*2
	elseif VCT.PlayerClass=="DEATHKNIGHT" and UnitBuff("player", "Blood Presence") then
		_,_,_,_,rank = GetTalentInfo(1, 14)
		return rank*3
	end
	return 0
end

function VCT:NumberCrunch()
	if VCT_Var.TargetLevel == 0 then
		if UnitLevel("target") == -1 then
			LevDiff=3
		elseif UnitLevel("target") == 0 then
			LevDiff=0		
		else
			LevDiff=UnitLevel("target")-UnitLevel("player")
		end	
	else
		LevDiff=VCT_Var.TargetLevel-UnitLevel("player")
	end
	
	CTMiss=5-(LevDiff*0.2)
	if CTMiss<0 then CTMiss = 0 end
	if VCT.PlayerRace == "NightElf" then CTMiss=CTMiss+2 end
	CTCov=100-CTMiss
	if CTCov<0 then CTCov = 0 end
		
	CTDodge=GetDodgeChance()-(LevDiff*0.2)
	if CTDodge<0 then CTDodge = 0 end
	if CTDodge>CTCov then
		CTDodge=CTCov
		CTCov=0
	else
		CTCov=CTCov-CTDodge
	end
		
	CTParry=GetParryChance()-(LevDiff*0.2)
	if CTParry<0 then CTParry = 0 end
	if CTParry>CTCov then
		CTParry=CTCov
		CTCov=0
	else
		CTCov=CTCov-CTParry
	end
		
	CTBlock=GetBlockChance()-(LevDiff*0.2)
	if not (IsEquippedItemType(locShields)) then CTBlock=0 end
	if CTBlock<0 then CTBlock = 0 end
	if CTBlock>CTCov then
		CTBlock=CTCov
		CTCov=0
	else
		CTCov=CTCov-CTBlock
	end
		
	CTCrit=5+(LevDiff*0.2)
	CTCrit=CTCrit-VCT:LowerCrit()
	if CTCrit<0 then CTCrit=0 end
	if CTCrit>CTCov then
		CTCrit=CTCov
		CTCov=0
	else
		CTCov=CTCov-CTCrit
	end
		
	if LevDiff>=4 then CTCrush = LevDiff*5*2-15 else CTCrush = 0 end
	if CTCrush<0 then CTCrush = 0 end
	if CTCrush>CTCov then
		CTCrush=CTCov
		CTCov=0
	else
		CTCov=CTCov-CTCrush
	end
	
	CTHit = 100-CTMiss-CTDodge-CTParry-CTBlock-CTCrush-CTCrit
	if CTHit < 0 then CTHit = 0 end
	CTCBlockChance = 0	
	if (VCT.PlayerClass == "PALADIN" and UnitBuff("player", "Holy Shield")) then CTBlockValue = 0.5
		elseif VCT.PlayerClass == "WARRIOR" then
			if GetPrimaryTalentTree()==3 then CTCBlockChance = GetMastery()*1.5 end
			if UnitBuff("player", "Hold the Line") then CTCBlockChance = CTCBlockChance+10 end
			CTBlockValue = 0.3
		else
			CTBlockValue = 0.3
	end

	CTMitigation = (CTBlock/100*(100-CTCBlockChance)*(1-CTBlockValue)) + (CTBlock*CTCBlockChance/100*0.4) + CTCrush*1.5 + CTCrit*2 + CTHit*1
	_, CTArmorDR = UnitArmor("player")
	if ((UnitLevel("player")+LevDiff)>1 and (UnitLevel("player")+LevDiff)<=59) then
		CTArmorDR = CTArmorDR/(CTArmorDR+85*(UnitLevel("player")+LevDiff)+400)
	elseif ((UnitLevel("player")+LevDiff)>=60 and (UnitLevel("player")+LevDiff)<=80) then
		CTArmorDR = CTArmorDR/(CTArmorDR+467.5*(UnitLevel("player")+LevDiff)-22167.5)
	else
		CTArmorDR = CTArmorDR/(CTArmorDR+2167.5*(UnitLevel("player")+LevDiff)-158167.5)
	end
end


function VCT:UpdateFrame()
	VCTFill = 4
	VCT.NumberCrunch()
	if VCT_Var.AlwaysShowTooltip and not VCTTooltip:IsVisible() then VCTOnEnter() end
	if (CTMiss+CTDodge+CTParry)>0 then
		VCT.Textures.GTexture:ClearAllPoints()
		if VCTFill+((CTMiss+CTDodge+CTParry)*((VCT_Var.Width-8)/100))<(VCT_Var.Width-4) then
			VCT.Textures.GTexture:SetWidth((CTMiss+CTDodge+CTParry)*((VCT_Var.Width-8)/100))
		else
			VCT.Textures.GTexture:SetWidth((VCT_Var.Width-4)-VCTFill)
		end
		VCT.Textures.GTexture:SetHeight(VCT.MainFrame:GetHeight()-6)
		VCT.Textures.GTexture:SetPoint("TOPLEFT", VCT.MainFrame, VCTFill, -3)
		VCTFill=VCTFill+VCT.Textures.GTexture:GetWidth()
		VCT.Textures.GTexture:Show()
		if VCTFill>(VCT_Var.Width-4) then VCTFill=(VCT_Var.Width-4) end
	end
	
	if CTBlock>0 and VCTFill<(VCT_Var.Width-4) then
		VCT.Textures.YTexture:ClearAllPoints()
		if VCTFill+(CTBlock*((VCT_Var.Width-8)/100))<(VCT_Var.Width-4) then
			VCT.Textures.YTexture:SetWidth(CTBlock*((VCT_Var.Width-8)/100))
		else
			VCT.Textures.YTexture:SetWidth((VCT_Var.Width-4)-VCTFill)
		end
		VCT.Textures.YTexture:SetHeight(VCT.MainFrame:GetHeight()-6)
		VCT.Textures.YTexture:SetPoint("TOPLEFT", VCT.MainFrame, VCTFill, -3)
		VCTFill=VCTFill+VCT.Textures.YTexture:GetWidth()
		VCT.Textures.YTexture:Show()
		if VCTFill>(VCT_Var.Width-4) then VCTFill=(VCT_Var.Width-4) end
	else
		VCT.Textures.YTexture:Hide()
	end
	
	if CTCrit>0 and VCTFill<(VCT_Var.Width-4) then
		VCT.Textures.RTexture:ClearAllPoints()
		if VCTFill+(CTCrit*((VCT_Var.Width-8)/100))<(VCT_Var.Width-4) then
			VCT.Textures.RTexture:SetWidth(CTCrit*((VCT_Var.Width-8)/100))
		else
			VCT.Textures.RTexture:SetWidth((VCT_Var.Width-4)-VCTFill)
		end
		VCT.Textures.RTexture:SetHeight(VCT.MainFrame:GetHeight()-6)
		VCT.Textures.RTexture:SetPoint("TOPLEFT", VCT.MainFrame, VCTFill, -3)
		VCTFill=VCTFill+VCT.Textures.RTexture:GetWidth()
		VCT.Textures.RTexture:Show()
		if VCTFill>(VCT_Var.Width-4) then VCTFill=(VCT_Var.Width-4) end
	else
		VCT.Textures.RTexture:Hide()
	end
	
	if CTCrush>0 and VCTFill<(VCT_Var.Width-4) then
		VCT.Textures.PTexture:ClearAllPoints()
		if VCTFill+(CTCrush*((VCT_Var.Width-8)/100))<(VCT_Var.Width-4) then
			VCT.Textures.PTexture:SetWidth(CTCrush*((VCT_Var.Width-8)/100))
		else
			VCT.Textures.PTexture:SetWidth((VCT_Var.Width-4)-VCTFill)
		end
		VCT.Textures.PTexture:SetHeight(VCT.MainFrame:GetHeight()-6)
		VCT.Textures.PTexture:SetPoint("TOPLEFT", VCT.MainFrame, VCTFill, -3)
		VCTFill=VCTFill+VCT.Textures.PTexture:GetWidth()
		VCT.Textures.PTexture:Show()
		if VCTFill>(VCT_Var.Width-4) then VCTFill=(VCT_Var.Width-4) end
	else
		VCT.Textures.PTexture:Hide()
	end
	
	if CTHit>0 and VCTFill<(VCT_Var.Width-4) then
		VCT.Textures.OTexture:ClearAllPoints()
		if VCTFill+(CTHit*((VCT_Var.Width-8)/100))<(VCT_Var.Width-4) then
			VCT.Textures.OTexture:SetWidth(CTHit*((VCT_Var.Width-8)/100))
		else
			VCT.Textures.OTexture:SetWidth((VCT_Var.Width-4)-VCTFill)
		end
		VCT.Textures.OTexture:SetHeight(VCT.MainFrame:GetHeight()-6)
		VCT.Textures.OTexture:SetPoint("TOPLEFT", VCT.MainFrame, VCTFill, -3)
		VCT.Textures.OTexture:Show()
	else
		VCT.Textures.OTexture:Hide()
	end
	if VCTTooltip:IsShown() then VCTOnEnter(self) end
end

function VCTEventHandler(self, event, eventname)
	if event == "ADDON_LOADED" and eventname == "VisualCombatTable" then
		VCT:LoadDefaults(VCT_Var, VCT_DefaultVars)
		VCT.MainFrame:UnregisterEvent("ADDON_LOADED")
		VCT:Loadup()
	end
end

do
	VCT.MainFrame = CreateFrame("Frame", nil, UIParent)
	VCT.MainFrame:RegisterEvent("ADDON_LOADED")
	VCT.MainFrame:SetScript("OnEvent", VCTEventHandler)
end